/*
    Copyright (C) 2010  ABRT team
    Copyright (C) 2010  RedHat Inc

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
#include "abrtlib.h"
#include "RPM.h"
#include "comm_layer_inner.h"

CRPM::CRPM()
{
    static const char *const argv[] = { "", NULL };
    m_poptContext = rpmcliInit(1, (char**)argv, NULL);
}

CRPM::~CRPM()
{
    rpmFreeRpmrc();
    rpmcliFini(m_poptContext);
}

void CRPM::LoadOpenGPGPublicKey(const char* pFileName)
{
    uint8_t* pkt = NULL;
    size_t pklen;
    pgpKeyID_t keyID;
    if (pgpReadPkts(pFileName, &pkt, &pklen) != PGPARMOR_PUBKEY)
    {
        free(pkt);
        error_msg("Can't load public GPG key %s", pFileName);
        return;
    }
    if (pgpPubkeyFingerprint(pkt, pklen, keyID) == 0)
    {
        char* fedoraFingerprint = pgpHexStr(keyID, sizeof(keyID));
        if (fedoraFingerprint != NULL)
        {
            m_setFingerprints.insert(fedoraFingerprint);
        }
    }
    free(pkt);
}

bool CRPM::CheckFingerprint(const char* pPackage)
{
    bool ret = false;
    rpmts ts = rpmtsCreate();
    rpmmi iter = rpmtsInitIterator(ts, RPMTAG_NAME, pPackage, 0);
    Header header = rpmmiNext(iter);

    if (header != NULL)
    {
	char *pgpsig = headerSprintf(
		header,
		"%|DSAHEADER?{%{DSAHEADER:pgpsig}}:{%|RSAHEADER?{%{RSAHEADER:pgpsig}}:"
		"{%|SIGGPG?{%{SIGGPG:pgpsig}}:{%|SIGPGP?{%{SIGPGP:pgpsig}}:{(none)}|}|}|}|",
		NULL,
		NULL,
		NULL);
	if(pgpsig) {
	    std::string PGPSignatureText = pgpsig;
	    free(pgpsig);

	    size_t Key_ID_pos = PGPSignatureText.find(" Key ID ");
	    if (Key_ID_pos != std::string::npos)
	    {
		std::string headerFingerprint = PGPSignatureText.substr(Key_ID_pos + sizeof (" Key ID ") - 1);

		if (headerFingerprint != "")
		{
		    if (m_setFingerprints.find(headerFingerprint) != m_setFingerprints.end())
		    {
			ret = true;
		    }
		}
	    }
	}
    }
    rpmmiFree(iter);
    rpmtsFree(ts);
    return ret;
}

bool CheckHash(const char* pPackage, const char* pPath)
{
    bool ret = false;
    rpmts ts = rpmtsCreate();
    rpmmi iter = rpmtsInitIterator(ts, RPMTAG_NAME, pPackage, 0);
    Header header = rpmmiNext(iter);
    if (header != NULL)
    {
        rpmfi fi = rpmfiNew(ts, header, RPMTAG_BASENAMES, 0);
        pgpHashAlgo hashAlgo;
        std::string headerHash;
        char computedHash[1024] = "";
	const unsigned char *digest = NULL;
	char * pgpHex = NULL;
	size_t lenp;
	struct stat sb;
	size_t digestlen = 0;
	char buf[8*BUFSIZ];
	size_t nb;
	int xx;

        while (rpmfiNext(fi) != -1)
        {
            if (strcmp(pPath, rpmfiFN(fi)) == 0)
            {
                digest = rpmfiDigest(fi, reinterpret_cast<int*>(&hashAlgo), &lenp);
		pgpHex = pgpHexStr(digest, lenp);
		headerHash = std::string((const char*)digest, lenp*2);

		if(Stat(pPath, &sb) >= 0) {
		    DIGEST_CTX ctx = rpmDigestInit(hashAlgo, RPMDIGEST_NONE);
		    FD_t fd = Fopen(pPath, "r");

		    while ((nb = Fread(buf, sizeof(buf[0]), sizeof(buf), fd)) > 0)
			xx = rpmDigestUpdate(ctx, buf, nb);
		    xx = Fclose(fd);    fd = NULL;
		    xx = rpmDigestFinal(ctx, &digest, &digestlen, 0);
		    pgpHex = pgpHexStr(digest, digestlen);
		    snprintf(computedHash, digestlen*2, "%s", pgpHex);
		}
                ret = (headerHash != "" && headerHash == computedHash);
                break;
            }
        }
        rpmfiFree(fi);
    }
    rpmmiFree(iter);
    rpmtsFree(ts);
    return ret;
}

std::string GetDescription(const char* pPackage)
{
    std::string pDescription;
    rpmts ts = rpmtsCreate();
    rpmmi iter = rpmtsInitIterator(ts, RPMTAG_NAME, pPackage, 0);
    Header header = rpmmiNext(iter);
    if (header != NULL)
    {
	HE_t he = (HE_t)memset(alloca(sizeof(*he)), 0, sizeof(*he));
	he->tag = RPMTAG_SUMMARY;
        if(headerGet(header, he, 0))
	{
	    pDescription += he->p.str;
	    _free(he->p.ptr);
	}
	pDescription += "\n\n";
	he->tag = RPMTAG_DESCRIPTION;
	if(headerGet(header, he, 0))
	{
	    pDescription += he->p.str;
	    _free(he->p.ptr);
	}
    }
    rpmmiFree(iter);
    return pDescription;
}

std::string GetComponent(const char* pFileName)
{
    std::string ret;
    char *package_name;
    rpmts ts = rpmtsCreate();
    rpmmi iter = rpmtsInitIterator(ts, RPMTAG_BASENAMES, pFileName, 0);
    Header header = rpmmiNext(iter);
    if (header != NULL)
    {
	HE_t he = (HE_t)memset(alloca(sizeof(*he)), 0, sizeof(*he));
	he->tag = RPMTAG_SOURCERPM;
        if(headerGet(header, he, 0))
        {
            package_name = get_package_name_from_NVR_or_NULL(he->p.str);
            ret = std::string(package_name);
            _free(he->p.ptr);
        }
    }

    rpmmiFree(iter);
    rpmtsFree(ts);
    return ret;
}

char* GetPackage(const char* pFileName)
{
    char* ret = NULL;
    rpmts ts = rpmtsCreate();
    rpmmi iter = rpmtsInitIterator(ts, RPMTAG_BASENAMES, pFileName, 0);
    Header header = rpmmiNext(iter);
    if (header != NULL)
    {
        ret = headerSprintf(header, "%{name}-%{version}-%{release}", NULL, NULL, NULL);
    }

    rpmmiFree(iter);
    rpmtsFree(ts);
    return ret;
}
